
library(MLmetrics)
library(psych)
library(glmnet)
library(pROC)
library(missForest)
library(caret)
library(caTools)
library(survival)
library(stir)
library(glmnetUtils)
library(randomForest)
library(e1071)

############################################################################
# re-running Cannon model
############################################################################

x <- read.csv("Moore_Psychosis_Risk_Prediction.csv")
x <- x[which(is.na(x$psy_T2_or_T3) == FALSE),]
x <- x[which(x$PS_GO1 == 1),]
n <- nrow(x)

results <- matrix(NA,10000,12)

temp <- x[,c("duration","smry_prime_tot","Age","Fam_Hx_Strict","vmem_a_ar_z","trauma_T1","gaf001","Overall_Speed")]
temp[,4] <- as.factor(temp[,4])
temp[,3] <- temp[,3]/12
set.seed(2)
temp <- missForest(temp)$ximp
x[,c("duration","smry_prime_tot","Age","Fam_Hx_Strict","vmem_a_ar_z","trauma_T1","gaf001","Overall_Speed")] <- temp

mod <- coxph(Surv(time=x$duration,event=x$psy_T2_or_T3)~x$smry_prime_tot+x$Age+x$Fam_Hx_Strict+x$vmem_a_ar_z+x$trauma_T1+x$gaf001+x$Overall_Speed,na.action=na.exclude)
pred1 <- predict(mod,na.action=na.exclude)
thresh1 <- pROC::coords(pROC::roc(x$psy_T2_or_T3~as.numeric(pred1)),x="best")[1]

for (j in 1:length(pred1)) {if (pred1[j] < thresh1) {pred1[j] <- 0}
if (pred1[j] > thresh1) {pred1[j] <- 1}}

for_later1 <- x$psy_T2_or_T3
for_later2 <- pred1

plot(pROC::roc(x$psy_T2_or_T3~as.numeric(pred1)),grid=TRUE,col="black",lwd=7)

for (dd in 1:10000) {

set.seed(dd+1)
key <- as.data.frame(cbind(runif(n),rep(1:2,n/2)))
key <- key[order(key$V1),]
thresh <- NA

for (i in 1:2) {

xtr <- x[key[,2] != i,]
xte  <- x[key[,2] == i,]
ytr <- xtr$psy_T2_or_T3      # - 1
yte <- xte$psy_T2_or_T3      # - 1
xtr <- xtr[,2:ncol(xtr)]
xte  <- xte[,2:ncol(xte)]

mod <- coxph(Surv(time=duration,event=ytr)~smry_prime_tot+Age+Fam_Hx_Strict+vmem_a_ar_z+trauma_T1+gaf001+Overall_Speed,data=xtr,na.action=na.exclude)

probs <- predict(mod,na.action=na.exclude)

thresh1 <- pROC::coords(pROC::roc(ytr~as.numeric(probs)),x="best")[1]

mod1 <- coxph(Surv(time=duration,event=yte)~smry_prime_tot+Age+Fam_Hx_Strict+vmem_a_ar_z+trauma_T1+gaf001+Overall_Speed,data=xte,na.action=na.exclude)
pred1 <- predict(mod1, newdata=as.data.frame(xte))

for (j in 1:length(pred1)) {if (pred1[j] < thresh1) {pred1[j] <- 0}
if (pred1[j] > thresh1) {pred1[j] <- 1}}

if (i == 1) {pred11 <- cbind(pred1,yte)}
if (i == 2) {pred21 <- cbind(pred1,yte)}

}

pred1 <- rbind(pred11,pred21)

plot(pROC::roc(pred1[,2]~pred1[,1]),grid=TRUE,col="green",lwd=1,add=TRUE)

results[dd,1] <- MLmetrics::AUC(pred1[,1], pred1[,2])
results[dd,2:12] <- confusionMatrix(data=as.factor(pred1[,1]), as.factor(pred1[,2]))$byClass

}

plot(pROC::roc(for_later1~as.numeric(for_later2)),grid=TRUE,col="black",lwd=7,add=TRUE,identity=TRUE,identity.lwd=3,identity.col="black",identity.lty=2)

write.csv(results,"Cannon_calculator.csv",na="")
